@@ -1,3 +1,4 @@
+# PYTHON_ARGCOMPLETE_OK
 """Migrating IPython < 4.0 to Jupyter
 
 This *copies* configuration and resources to their new locations in Jupyter
@@ -22,11 +23,14 @@ Migrations:
 
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
+from __future__ import annotations
 
 import os
 import re
 import shutil
-from datetime import datetime
+from datetime import datetime, timezone
+from pathlib import Path
+from typing import Any
 
 from traitlets.config.loader import JSONFileConfigLoader, PyFileConfigLoader
 from traitlets.log import get_logger
@@ -35,20 +39,21 @@ from .application import JupyterApp
 from .paths import jupyter_config_dir, jupyter_data_dir
 from .utils import ensure_dir_exists
 
-pjoin = os.path.join
+# mypy: disable-error-code="no-untyped-call"
+
 
 migrations = {
-    pjoin("{ipython_dir}", "nbextensions"): pjoin("{jupyter_data}", "nbextensions"),
-    pjoin("{ipython_dir}", "kernels"): pjoin("{jupyter_data}", "kernels"),
-    pjoin("{profile}", "nbconfig"): pjoin("{jupyter_config}", "nbconfig"),
+    str(Path("{ipython_dir}", "nbextensions")): str(Path("{jupyter_data}", "nbextensions")),
+    str(Path("{ipython_dir}", "kernels")): str(Path("{jupyter_data}", "kernels")),
+    str(Path("{profile}", "nbconfig")): str(Path("{jupyter_config}", "nbconfig")),
 }
 
-custom_src_t = pjoin("{profile}", "static", "custom")
-custom_dst_t = pjoin("{jupyter_config}", "custom")
+custom_src_t = str(Path("{profile}", "static", "custom"))
+custom_dst_t = str(Path("{jupyter_config}", "custom"))
 
 for security_file in ("notebook_secret", "notebook_cookie_secret", "nbsignatures.db"):
-    src = pjoin("{profile}", "security", security_file)
-    dst = pjoin("{jupyter_data}", security_file)
+    src = str(Path("{profile}", "security", security_file))
+    dst = str(Path("{jupyter_data}", security_file))
     migrations[src] = dst
 
 config_migrations = ["notebook", "nbconvert", "qtconsole"]
@@ -64,7 +69,7 @@ config_substitutions = {
 }
 
 
-def get_ipython_dir():
+def get_ipython_dir() -> str:
     """Return the IPython directory location.
 
     Not imported from IPython because the IPython implementation
@@ -75,67 +80,68 @@ def get_ipython_dir():
     We only need to support the IPython < 4 behavior for migration,
     so importing for forward-compatibility and edge cases is not important.
     """
-    return os.environ.get("IPYTHONDIR", os.path.expanduser("~/.ipython"))
+    return os.environ.get("IPYTHONDIR", str(Path("~/.ipython").expanduser()))
 
 
-def migrate_dir(src, dst):
+def migrate_dir(src: str, dst: str) -> bool:
     """Migrate a directory from src to dst"""
     log = get_logger()
-    if not os.listdir(src):
+    src_path = Path(src)
+    dst_path = Path(dst)
+    if not any(src_path.iterdir()):
         log.debug("No files in %s", src)
         return False
-    if os.path.exists(dst):
-        if os.listdir(dst):
+    if dst_path.exists():
+        if any(dst_path.iterdir()):
             # already exists, non-empty
             log.debug("%s already exists", dst)
             return False
-        else:
-            os.rmdir(dst)
+        dst_path.rmdir()
     log.info("Copying %s -> %s", src, dst)
-    ensure_dir_exists(os.path.dirname(dst))
+    ensure_dir_exists(dst_path.parent)
     shutil.copytree(src, dst, symlinks=True)
     return True
 
 
-def migrate_file(src, dst, substitutions=None):
+def migrate_file(src: str | Path, dst: str | Path, substitutions: Any = None) -> bool:
     """Migrate a single file from src to dst
 
     substitutions is an optional dict of {regex: replacement} for performing replacements on the file.
     """
     log = get_logger()
-    if os.path.exists(dst):
+    dst_path = Path(dst)
+    if dst_path.exists():
         # already exists
         log.debug("%s already exists", dst)
         return False
     log.info("Copying %s -> %s", src, dst)
-    ensure_dir_exists(os.path.dirname(dst))
+    ensure_dir_exists(dst_path.parent)
     shutil.copy(src, dst)
     if substitutions:
-        with open(dst, encoding="utf-8") as f:
+        with dst_path.open() as f:
             text = f.read()
         for pat, replacement in substitutions.items():
             text = pat.sub(replacement, text)
-        with open(dst, "w", encoding="utf-8") as f:
+        with dst_path.open("w") as f:
             f.write(text)
     return True
 
 
-def migrate_one(src, dst):
+def migrate_one(src: str, dst: str) -> bool:
     """Migrate one item
 
     dispatches to migrate_dir/_file
     """
     log = get_logger()
-    if os.path.isfile(src):
+    if Path(src).is_file():
         return migrate_file(src, dst)
-    elif os.path.isdir(src):
+    if Path(src).is_dir():
         return migrate_dir(src, dst)
-    else:
-        log.debug("Nothing to migrate for %s", src)
-        return False
+    log.debug("Nothing to migrate for %s", src)
+    return False
 
 
-def migrate_static_custom(src, dst):
+def migrate_static_custom(src: str, dst: str) -> bool:
     """Migrate non-empty custom.js,css from src to dst
 
     src, dst are 'custom' directories containing custom.{js,css}
@@ -143,12 +149,12 @@ def migrate_static_custom(src, dst):
     log = get_logger()
     migrated = False
 
-    custom_js = pjoin(src, "custom.js")
-    custom_css = pjoin(src, "custom.css")
+    custom_js = Path(src, "custom.js")
+    custom_css = Path(src, "custom.css")
     # check if custom_js is empty:
     custom_js_empty = True
-    if os.path.isfile(custom_js):
-        with open(custom_js, encoding="utf-8") as f:
+    if Path(custom_js).is_file():
+        with Path.open(custom_js, encoding="utf-8") as f:
             js = f.read().strip()
             for line in js.splitlines():
                 if not (line.isspace() or line.strip().startswith(("/*", "*", "//"))):
@@ -157,8 +163,8 @@ def migrate_static_custom(src, dst):
 
     # check if custom_css is empty:
     custom_css_empty = True
-    if os.path.isfile(custom_css):
-        with open(custom_css, encoding="utf-8") as f:
+    if Path(custom_css).is_file():
+        with Path.open(custom_css, encoding="utf-8") as f:
             css = f.read().strip()
             custom_css_empty = css.startswith("/*") and css.endswith("*/")
 
@@ -175,24 +181,22 @@ def migrate_static_custom(src, dst):
     if not custom_js_empty or not custom_css_empty:
         ensure_dir_exists(dst)
 
-    if not custom_js_empty:
-        if migrate_file(custom_js, pjoin(dst, "custom.js")):
-            migrated = True
-    if not custom_css_empty:
-        if migrate_file(custom_css, pjoin(dst, "custom.css")):
-            migrated = True
+    if not custom_js_empty and migrate_file(custom_js, Path(dst, "custom.js")):
+        migrated = True
+    if not custom_css_empty and migrate_file(custom_css, Path(dst, "custom.css")):
+        migrated = True
 
     return migrated
 
 
-def migrate_config(name, env):
+def migrate_config(name: str, env: Any) -> list[Any]:
     """Migrate a config file.
 
     Includes substitutions for updated configurable names.
     """
     log = get_logger()
-    src_base = pjoin("{profile}", "ipython_{name}_config").format(name=name, **env)
-    dst_base = pjoin("{jupyter_config}", "jupyter_{name}_config").format(name=name, **env)
+    src_base = str(Path(f"{env['profile']}", f"ipython_{name}_config"))
+    dst_base = str(Path(f"{env['jupyter_config']}", f"jupyter_{name}_config"))
     loaders = {
         ".py": PyFileConfigLoader,
         ".json": JSONFileConfigLoader,
@@ -201,7 +205,7 @@ def migrate_config(name, env):
     for ext in (".py", ".json"):
         src = src_base + ext
         dst = dst_base + ext
-        if os.path.exists(src):
+        if Path(src).exists():
             cfg = loaders[ext](src).load_config()
             if cfg:
                 if migrate_file(src, dst, substitutions=config_substitutions):
@@ -212,21 +216,20 @@ def migrate_config(name, env):
     return migrated
 
 
-def migrate():
+def migrate() -> bool:
     """Migrate IPython configuration to Jupyter"""
     env = {
         "jupyter_data": jupyter_data_dir(),
         "jupyter_config": jupyter_config_dir(),
         "ipython_dir": get_ipython_dir(),
-        "profile": os.path.join(get_ipython_dir(), "profile_default"),
+        "profile": str(Path(get_ipython_dir(), "profile_default")),
     }
     migrated = False
     for src_t, dst_t in migrations.items():
         src = src_t.format(**env)
         dst = dst_t.format(**env)
-        if os.path.exists(src):
-            if migrate_one(src, dst):
-                migrated = True
+        if Path(src).exists() and migrate_one(src, dst):
+            migrated = True
 
     for name in config_migrations:
         if migrate_config(name, env):
@@ -235,14 +238,13 @@ def migrate():
     custom_src = custom_src_t.format(**env)
     custom_dst = custom_dst_t.format(**env)
 
-    if os.path.exists(custom_src):
-        if migrate_static_custom(custom_src, custom_dst):
-            migrated = True
+    if Path(custom_src).exists() and migrate_static_custom(custom_src, custom_dst):
+        migrated = True
 
     # write a marker to avoid re-running migration checks
     ensure_dir_exists(env["jupyter_config"])
-    with open(os.path.join(env["jupyter_config"], "migrated"), "w", encoding="utf-8") as f:
-        f.write(datetime.utcnow().isoformat())
+    with Path.open(Path(env["jupyter_config"], "migrated"), "w", encoding="utf-8") as f:
+        f.write(datetime.now(tz=timezone.utc).isoformat())
 
     return migrated
 
@@ -267,7 +269,7 @@ class JupyterMigrate(JupyterApp):
     If the destinations already exist, nothing will be done.
     """
 
-    def start(self):
+    def start(self) -> None:
         """Start the application."""
         if not migrate():
             self.log.info("Found nothing to migrate.")
